/**
 * Copyright (c) Members of the EGEE Collaboration. 2007-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2007-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>


#include "daemonize.h"

pid_t daemonize(void) 
{
    pid_t pid;

    pid=fork();
    switch (pid) 
    {
        case 0:
            break;
        case -1:
           /* Error */
           fprintf(stderr, "Error daemonizing (fork)! %d - %s\n", errno, strerror(errno));
           exit(1);
        default:
	   return pid;
/*           _exit(0);*/
    }

    if (setsid() == 0) 
    {
        fprintf(stderr, "Error daemonizing (setsid)! %d - %s\n", errno, strerror(errno));
        return -1;
    }

#if 0
    switch (fork()) 
    {
        case 0:
            break;
        case -1:
            /* Error */
            fprintf(stderr, "Error demonizing (fork2)! %d - %s\n", errno, strerror(errno));
            exit(0);
            break;
        default:
            _exit(0);
    }
#endif

    if (chdir("/"))
	fprintf(stderr, "Cannot chdir to /, will continue in cwd: %s\n",
		strerror(errno));

    /* redirect std{in,out,err} */
    if (freopen("/dev/null","a",stdout)==NULL ||
        freopen("/dev/null","a",stderr)==NULL ||
        freopen("/dev/null","r",stdin)==NULL)
    {
        fprintf(stderr,"%s: Cannot reattach stdin/stderr/stdout: %s\n",
		__func__, strerror(errno));
    }
    
    return 0;
}


